# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import os
import signal
import sys

from time import sleep

from celery.exceptions import SoftTimeLimitExceeded
from celery.utils.log import get_task_logger

from .app import app

E_MARKER_DELAY_ERROR = """\
Retrying marker.delay().  It failed to start: {0}\
"""

logger = get_task_logger(__name__)


def marker(s, sep='-'):
    """Marker is a task that logs something to the worker logs.

    :param s: Text to log.

    """
    print('{0}{1}'.format(sep, s))
    while True:
        try:
            return _marker.delay(s, sep)
        except Exception as exc:
            print(E_MARKER_DELAY_ERROR.format(exc))


@app.task
def _marker(s, sep='---'):
    print('{sep} {0} {sep}'.format(s, sep=sep))


@app.task
def add(x, y):
    """Add two numbers."""
    return x + y


@app.task(bind=True)
def ids(self, i):
    """Returns a tuple of ``root_id``, ``parent_id`` and
    the argument passed as ``i``."""
    return self.request.root_id, self.request.parent_id, i


@app.task(bind=True)
def collect_ids(self, res, i):
    """Used as a callback in a chain or group where the previous tasks
    are :task:`ids`: returns a tuple of::

        (previous_result, (root_id, parent_id, i))

    """
    return res, ids(i)


@app.task
def xsum(x):
    """Takes a list of numbers and returns the total."""
    return sum(x)


@app.task
def any_(*args, **kwargs):
    """Task taking any argument, returning nothing.

    This is useful for testing related to large arguments: big values,
    an insane number of positional arguments, etc.

    :keyword sleep: Optional number of seconds to sleep for before returning.

    """
    wait = kwargs.get('sleep')
    if wait:
        sleep(wait)


@app.task
def any_returning(*args, **kwargs):
    """The same as :task:`any` except it returns the arguments given
    as a tuple of ``(args, kwargs)``."""
    any_(*args, **kwargs)
    return args, kwargs


@app.task
def exiting(status=0):
    """Task calling ``sys.exit(status)`` to terminate its own worker
    process."""
    sys.exit(status)


@app.task
def kill(sig=getattr(signal, 'SIGKILL', None) or signal.SIGTERM):
    """Task sending signal to process currently executing itself.

    :keyword sig: Signal to send as signal number, default is :sig:`KILL`
      on platforms that supports that signal, for other platforms (i.e Windows)
      it will be :sig:`TERM`.

    """
    os.kill(os.getpid(), sig)


@app.task
def sleeping(i, **_):
    """Task sleeping for ``i`` seconds, and returning nothing."""
    sleep(i)


@app.task
def sleeping_ignore_limits(i, **_):
    """Task sleeping for ``i`` seconds, while ignoring soft time limits.

    If the task is signalled with
    :exc:`~celery.exceptions.SoftTimeLimitExceeded` the signal is ignored
    and the task will sleep for ``i`` seconds again, which will trigger
    the hard time limit (if enabled).

    """
    try:
        sleep(i)
    except SoftTimeLimitExceeded:
        sleep(i)


@app.task(bind=True)
def retries(self, n=1, countdown=1, return_value=10):
    """Task that retries itself ``n`` times.

    :param n: Number of times to retry.
    :param n: Seconds to wait (``int``/``float``) between each retry (default
      is one second).
    :param return_value: Value to return when task finally succeeds.
        Default is 10 (don't ask, I guess it's a random true value).

    """
    if not self.request.retries or self.request.retries < n:
        raise self.retry(countdown=countdown)
    return return_value


@app.task
def print_unicode(log_message='håå®ƒ valmuefrø', print_message='hiöäüß'):
    """Task that both logs and print strings containing funny characters."""
    logger.warning(log_message)
    print(print_message)


@app.task
def segfault():
    """Task causing a segfault, abruptly terminating the process
    executing the task."""
    import ctypes
    ctypes.memset(0, 0, 1)
    assert False, 'should not get here'


@app.task(bind=True)
def chord_adds(self, x):
    """Task that adds a new task to the current chord in a workflow."""
    self.add_to_chord(add.s(x, x))
    return 42


@app.task(bind=True)
def chord_replace(self, x):
    """Task that replaces itself in the current chord."""
    return self.replace_in_chord(add.s(x, x))


@app.task
def raising(exc=KeyError()):
    """Task raising exception.

    :param exc: Exception to raise.

    """
    raise exc


@app.task
def errback(request, exc, traceback):
    print('Task id {0!r} raised exception: {1!r}\n{2}'.format(
        request.id, exc, traceback,
    ))


@app.task
def old_errback(task_id):
    print('Task id %r raised exception' % (task_id,))


@app.task
def logs(msg, p=False):
    """Log a message to the worker logs.

    :keyword p: If set to :const:`True` the message will be printed instead
      of logged, thus being redirected to the stdout logger.

    """
    print(msg) if p else logger.info(msg)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import os

from sphinx_celery import conf

globals().update(conf.build_config(
    'cyanide', __file__,
    project='Cyanide',
    # version_dev='2.0',
    # version_stable='1.4',
    canonical_url='https://cyanide.readthedocs.io',
    webdomain='',
    github_project='celery/cyanide',
    copyright='2013-2016',
    html_logo='images/logo.png',
    html_favicon='images/favicon.ico',
    html_prepend_sidebars=[],
    include_intersphinx={'python', 'sphinx'},
    # django_settings='testproj.settings',
    # path_additions=[os.path.join(os.pardir, 'testproj')],
    apicheck_package='cyanide',
     apicheck_ignore_modules=[
       'cyanide.__main__',
       'cyanide.bin',
       'cyanide.suites',
     ],
))



from __future__ import absolute_import, print_function, unicode_literals

import celery
import cyanide

import inspect
import platform
import socket
import sys
import traceback

from collections import OrderedDict, defaultdict, namedtuple
from functools import partial
from itertools import count, cycle

from celery.exceptions import TimeoutError
from celery.five import items, monotonic, range, values
from celery.utils.debug import blockdetection
from celery.utils.imports import qualname
from celery.utils.text import pluralize, truncate
from celery.utils.term import colored
from kombu.utils import retry_over_time

from .fbi import FBI
from .tasks import marker, _marker

try:
    from celery.platforms import isatty
except ImportError:
    from celery.utils import isatty

try:
    import celery.utils.time as timeutils
except ImportError:
    from celery.utils import timeutils

BANNER = """\
Cyanide v{version} [celery {celery_version}]

{platform}

[config]
.> app:    {app}
.> broker: {conninfo}
.> suite: {suite}

[toc: {total} {TESTS} total]
{toc}
"""

F_PROGRESS = """\
{0.index:2d}: {0.test.__name__:<36} {status} \
({0.iteration}/{0.total_iterations}) rep#{0.repeats} runtime: \
{runtime}/{elapsed}\
"""

E_STILL_WAITING = """\
Still waiting for {0}. Trying again {when}: {exc!r}\
"""

Progress = namedtuple('Progress', (
    'test', 'iteration', 'total_iterations',
    'index', 'repeats', 'runtime', 'elapsed', 'completed',
))


Inf = float('Inf')


def assert_equal(a, b):
    assert a == b, '{0!r} != {1!r}'.format(a, b)


class StopSuite(Exception):
    pass


class Sentinel(Exception):
    pass


def humanize_seconds(secs, prefix='', sep='', now='now', **kwargs):
    s = timeutils.humanize_seconds(secs, prefix, sep, now, **kwargs)
    if s == now and secs > 0:
        return '{prefix}{sep}{0:.2f} seconds'.format(
            float(secs), prefix=prefix, sep=sep)
    return s


def pstatus(p, status=None):
    runtime = format(monotonic() - p.runtime, '.4f')
    elapsed = format(monotonic() - p.elapsed, '.4f')
    return F_PROGRESS.format(
        p,
        status=status or '',
        runtime=humanize_seconds(runtime, now=runtime),
        elapsed=humanize_seconds(elapsed, now=elapsed),
    )


class Speaker(object):

    def __init__(self, gap=5.0, file=None):
        self.gap = gap
        self.file = sys.stdout if file is None else file
        self.last_noise = monotonic() - self.gap * 2

    def beep(self):
        now = monotonic()
        if now - self.last_noise >= self.gap:
            self.emit()
            self.last_noise = now

    def emit(self):
        print('\a', file=self.file, end='')


class Meter(object):

    def __init__(self, s='.', end='', cursor='/-\\', file=None):
        self.s = s
        self.end = end
        self.file = sys.stdout if file is None else file
        self.counter = 0
        self.cursor = cycle(cursor)

    def emit(self, *args, **kwargs):
        self.counter += len(self.s)
        print(self.s * (self.counter - 1) + next(self.cursor),
              end='\r', file=self.file)
        self.file.flush()

    def revert(self):
        self.counter = 0


def testgroup(*funs):
    return OrderedDict((fun.__name__, fun) for fun in funs)


class ManagerMixin(object):
    TaskPredicate = StopSuite

    def _init_manager(self, app,
                      block_timeout=30 * 60, stdout=None, stderr=None):
        self.stdout = sys.stdout if stdout is None else stdout
        self.stderr = sys.stderr if stderr is None else stderr
        self.connerrors = self.app.connection().recoverable_connection_errors
        self.block_timeout = block_timeout
        self.progress = None
        self.speaker = Speaker(file=self.stdout)
        self.fbi = FBI(app)

    def missing_results(self, r):
        return [res.id for res in r if res.id not in res.backend._cache]

    def wait_for(self, fun, catch,
                 desc='thing', args=(), kwargs={}, errback=None,
                 max_retries=10, interval_start=0.1, interval_step=0.5,
                 interval_max=5.0, emit_warning=False, **options):
        meter = Meter(file=self.stdout)

        def on_error(exc, intervals, retries):
            interval = next(intervals)
            if emit_warning:
                self.warn(E_STILL_WAITING.format(
                    desc, when=humanize_seconds(interval, 'in', ' '), exc=exc,
                ))
            else:
                meter.emit()
            if errback:
                errback(exc, interval, retries)
            return interval

        return self.retry_over_time(
            fun, catch,
            args=args, kwargs=kwargs,
            errback=on_error, max_retries=max_retries,
            interval_start=interval_start, interval_step=interval_step,
            **options
        )

    def ensure_not_for_a_while(self, fun, catch,
                               desc='thing', max_retries=20,
                               interval_start=0.1, interval_step=0.02,
                               interval_max=1.0, emit_warning=False,
                               **options):
        meter = Meter(file=self.stdout)
        try:
            return self.wait_for(
                fun, catch, desc=desc, max_retries=max_retries,
                interval_start=interval_start, interval_step=interval_step,
                interval_max=interval_max, emit_warning=emit_warning,
                errback=meter.emit,
            )
        except catch:
            pass
        else:
            raise AssertionError('Should not have happened: {0}'.format(desc))
        finally:
            meter.revert()

    def retry_over_time(self, *args, **kwargs):
        return retry_over_time(*args, **kwargs)

    def join(self, r, propagate=False, max_retries=10, **kwargs):
        if self.no_join:
            return
        received = []

        def on_result(task_id, value):
            received.append(task_id)

        for i in range(max_retries) if max_retries else count(0):
            received[:] = []
            try:
                return r.get(callback=on_result, propagate=propagate, **kwargs)
            except (socket.timeout, TimeoutError) as exc:
                waiting_for = self.missing_results(r)
                self.speaker.beep()
                marker(
                    'Still waiting for {0}/{1}: [{2}]: {3!r}'.format(
                        len(r) - len(received), len(r),
                        truncate(', '.join(waiting_for)), exc), '!',
                )
                self.fbi.diag(waiting_for)
            except self.connerrors as exc:
                self.speaker.beep()
                marker('join: connection lost: {0!r}'.format(exc), '!')
        raise self.TaskPredicate('Test failed: Missing task results')

    def inspect(self, timeout=1):
        return self.app.control.inspect(timeout=timeout)

    def query_tasks(self, ids, timeout=0.5):
        for reply in items(self.inspect(timeout).query_task(ids) or []):
            yield reply

    def query_task_states(self, ids, timeout=0.5):
        states = defaultdict(set)
        for hostname, reply in self.query_tasks(ids, timeout=timeout):
            for task_id, (state, _) in items(reply):
                states[state].add(task_id)
        return states

    def assert_accepted(self, ids, interval=0.5,
                        desc='waiting for tasks to be accepted', **policy):
        return self.assert_task_worker_state(
            self.is_accepted, ids, interval=interval, desc=desc, **policy
        )

    def assert_received(self, ids, interval=0.5,
                        desc='waiting for tasks to be received', **policy):
        return self.assert_task_worker_state(
            self.is_accepted, ids, interval=interval, desc=desc, **policy
        )

    def assert_task_worker_state(self, fun, ids, interval=0.5, **policy):
        return self.wait_for(
            partial(self.true_or_raise, fun, ids, timeout=interval),
            (Sentinel,), **policy
        )

    def is_received(self, ids, **kwargs):
        return self._ids_matches_state(
            ['reserved', 'active', 'ready'], ids, **kwargs)

    def is_accepted(self, ids, **kwargs):
        return self._ids_matches_state(['active', 'ready'], ids, **kwargs)

    def _ids_matches_state(self, expected_states, ids, timeout=0.5):
        states = self.query_task_states(ids, timeout=timeout)
        return all(
            any(t in s for s in [states[k] for k in expected_states])
            for t in ids
        )

    def true_or_raise(self, fun, *args, **kwargs):
        res = fun(*args, **kwargs)
        if not res:
            raise Sentinel()
        return res


class Suite(ManagerMixin):

    TaskPredicate = StopSuite

    def __init__(self, app, no_color=False, **kwargs):
        self.app = app
        self._init_manager(app, **kwargs)
        if not isatty(self.stdout):
            no_color = True
        self.colored = colored(enabled=not no_color)
        self.init_groups()

    def setup(self):
        pass

    def teardown(self):
        pass

    def print(self, message, file=None):
        print(message, file=self.stdout if file is None else file)

    def error(self, message):
        print(self.colored.red(message), file=self.stderr)

    def warn(self, message):
        print(self.colored.cyan(message), file=self.stdout)

    def init_groups(self):
        acc = defaultdict(list)
        for attr in dir(self):
            if not _is_descriptor(self, attr):
                meth = getattr(self, attr)
                try:
                    groups = meth.__func__.__testgroup__
                except AttributeError:
                    pass
                else:
                    for g in groups:
                        acc[g].append(meth)
        # sort the tests by the order in which they are defined in the class
        for g in values(acc):
            g[:] = sorted(g, key=lambda m: m.__func__.__testsort__)
        self.groups = dict(
            (name, testgroup(*tests)) for name, tests in items(acc)
        )

    def run(self, names=None, iterations=50, offset=0,
            numtests=None, list_all=False, repeat=0, group='all',
            diag=False, no_join=False, **kw):
        self.no_join = no_join
        self.fbi.enable(diag)
        tests = self.filtertests(group, names)[offset:numtests or None]
        if list_all:
            return self.print(self.testlist(tests))
        self.print(self.banner(tests))
        self.print('+enable worker task events...')
        self.app.control.enable_events()
        it = count() if repeat == Inf else range(int(repeat) or 1)
        for i in it:
            marker(
                '{0} (repetition {1})'.format(
                    self.colored.bold('suite start'), i + 1),
                '+',
            )
            for j, test in enumerate(tests):
                self.runtest(test, iterations, j + 1, i + 1)
            marker(
                '{0} (repetition {1})'.format(
                    self.colored.bold('suite end'), i + 1),
                '+',
            )

    def assert_equal(self, a, b):
        return assert_equal(a, b)

    def filtertests(self, group, names):
        tests = self.groups[group]
        try:
            return ([tests[n] for n in names] if names
                    else list(values(tests)))
        except KeyError as exc:
            raise KeyError('Unknown test name: {0}'.format(exc))

    def testlist(self, tests):
        return ',\n'.join(
            '.> {0}) {1}'.format(i + 1, t.__name__)
            for i, t in enumerate(tests)
        )

    def banner(self, tests):
        app = self.app
        return BANNER.format(
            app='{0}:0x{1:x}'.format(app.main or '__main__', id(app)),
            version=cyanide.__version__,
            celery_version=celery.VERSION_BANNER,
            conninfo=app.connection().as_uri(),
            platform=platform.platform(),
            toc=self.testlist(tests),
            TESTS=pluralize(len(tests), 'test'),
            total=len(tests),
            suite=':'.join(qualname(self).rsplit('.', 1)),
        )

    def runtest(self, fun, n=50, index=0, repeats=1):
        n = getattr(fun, '__iterations__', None) or n
        header = '[[[{0}({1})]]]'.format(fun.__name__, n)
        if repeats > 1:
            header = '{0} #{1}'.format(header, repeats)
        self.print(header)
        with blockdetection(self.block_timeout):
            with self.fbi.investigation():
                runtime = elapsed = monotonic()
                i = 0
                failed = False
                self.progress = Progress(
                    fun, i, n, index, repeats, elapsed, runtime, 0,
                )
                _marker.delay(pstatus(self.progress))

                try:
                    for i in range(n):
                        runtime = monotonic()
                        self.progress = Progress(
                            fun, i + 1, n, index, repeats, runtime, elapsed, 0,
                        )
                        self.execute_test(fun)

                except Exception:
                    failed = True
                    self.speaker.beep()
                    raise
                finally:
                    if n > 1 or failed:
                        self.print('{0} {1} iterations in {2}'.format(
                            'failed after' if failed else 'completed',
                            i + 1, humanize_seconds(monotonic() - elapsed),
                        ), file=self.stderr if failed else self.stdout)
                    if not failed:
                        self.progress = Progress(
                            fun, i + 1, n, index, repeats, runtime, elapsed, 1,
                        )

    def execute_test(self, fun):
        self.setup()
        try:
            try:
                fun()
            except StopSuite:
                raise
            except AssertionError as exc:
                self.on_test_error(exc, 'FAILED')
            except Exception as exc:
                self.on_test_error(exc, 'ERROR')
            else:
                self.print(pstatus(self.progress, self.colored.green('OK')))
        finally:
            self.teardown()

    def on_test_error(self, exc, status):
        self.error('-> {0!r}'.format(exc))
        self.error(traceback.format_exc())
        self.error(pstatus(self.progress, self.colored.red(status)))

    def dump_progress(self):
        return pstatus(self.progress) if self.progress else 'No test running'

_creation_counter = count(0)


def testcase(*groups, **kwargs):
    if not groups:
        raise ValueError('@testcase requires at least one group name')

    def _mark_as_case(fun):
        fun.__testgroup__ = groups
        fun.__testsort__ = next(_creation_counter)
        fun.__iterations__ = kwargs.get('iterations')
        return fun

    return _mark_as_case


def _is_descriptor(obj, attr):
    try:
        cattr = getattr(obj.__class__, attr)
    except AttributeError:
        pass
    else:
        return not inspect.ismethod(cattr) and hasattr(cattr, '__get__')
    return False



from __future__ import absolute_import, print_function, unicode_literals

import json
import os

from collections import OrderedDict

from celery.bin.base import Command, Option

from cyanide.vagrant import Vagrant, path


def csv_list_option(option, opt, value, parser):
    setattr(parser.values, option.dest, value.split(','))


class vagrant(Command):
    Vagrant = Vagrant

    def __init__(self, *args, **kwargs):
        super(vagrant, self).__init__(*args, **kwargs)
        self.commands = OrderedDict([
            ('up', self.up),
            ('status', self.status),
            ('sshargs', self.sshargs),
            ('destroy', self.destroy),
            ('suspend', self.suspend),
            ('resume', self.resume),
            ('halt', self.halt),
            ('provision', self.provision),
            ('reload', self.reload),
            ('conf', self.conf),
            ('user', self.user),
            ('hostname', self.hostname),
            ('user_hostname', self.user_hostname),
            ('user_hostname_port', self.user_hostname_port),
            ('port', self.port),
            ('keyfile', self.keyfile),
            ('box_list', self.box_list),
            ('path', self.path),
            ('statedir', self.statedir),
            ('snapshot_push', self.snapshot_push),
            ('snapshot_pop', self.snapshot_pop),
            ('snapshot_save', self.snapshot_save),
            ('snapshot_restore', self.snapshot_restore),
            ('snapshot_delete', self.snapshot_delete),
            ('snapshot_list', self.snapshot_list),
            ('version', self.version),
        ])
        self.vagrant = None

    def run(self, *args, **options):
        if not args:
            raise self.Error('missing subcommand. Try --help?')
        try:
            self.vagrant = self.create_session(**options)
            return self.run_command(*args, **options)
        except KeyboardInterrupt:
            print('interrupted by user: exiting...', file=self.stdout)

    def create_session(self, root, quiet_stdout, quiet_stderr, **kwargs):
        # make sure ``rm -rf $(celery vagrant statedir)`` does not
        # do horrible things.
        if root == '/':
            raise RuntimeError('Vagrant dir cannot be root (/) !!!')
        if '*' in root:
            raise RuntimeError('Vagrant dir cannot contain the * character')
        return self.Vagrant(
            root=root,
            quiet_stdout=quiet_stdout,
            quiet_stderr=quiet_stderr,
        )

    def usage(self, command):
        return '%prog {command} [{commands}] [options] {0.args}'.format(
            self, command=command, commands='|'.join(self.commands))

    def up(self, name, provision_with, **kwargs):
        self.vagrant.up(vm_name=name, provision_with=provision_with)

    def status(self, name, **kwargs):
        print(self.vagrant.status(vm_name=name), file=self.stdout)

    def sshargs(self, name, **kwargs):
        config = self.vagrant.conf(vm_name=name)
        print('{0}@{1} -p {2} -i {3}'.format(
            config['User'],
            config['HostName'],
            config['Port'],
            config['IdentityFile'],
        ), file=self.stdout)

    def destroy(self, name, **kwargs):
        self.vagrant.destroy(vm_name=name)

    def provision(self, name, provision_with, **kwargs):
        self.vagrant.provision(vm_name=name, provision_with=provision_with)

    def reload(self, name, provision_with, **kwargs):
        self.vagrant.reload(vm_name=name, provision_with=provision_with)

    def suspend(self, name, **kwargs):
        self.vagrant.suspend(vm_name=name)

    def resume(self, name, **kwargs):
        self.vagrant.resume(vm_name=name)

    def halt(self, name, force, **kwargs):
        self.vagrant.halt(vm_name=name, force=force)

    def conf(self, name, arguments, **kwargs):
        self.pretty_json(self.vagrant.conf(
            vm_name=name,
            ssh_config=arguments and arguments[0] or None,
        ))

    def path(self, **kwargs):
        print(self.vagrant.root, file=self.stdout)

    def statedir(self, **kwargs):
        print(os.path.join(self.vagrant.root, '.vagrant'), file=self.stdout)

    def user(self, name, **kwargs):
        print(self.vagrant.user(vm_name=name), file=self.stdout)

    def hostname(self, name, **kwargs):
        print(self.vagrant.hostname(vm_name=name), file=self.stdout)

    def user_hostname(self, name, **kwargs):
        print(self.vagrant.user_hostname(vm_name=name), file=self.stdout)

    def port(self, name, **kwargs):
        print(self.vagrant.port(vm_name=name), file=self.stdout)

    def user_hostname_port(self, name, **kwargs):
        print(self.vagrant.user_hostname_port(vm_name=name), file=self.stdout)

    def keyfile(self, name, **kwargs):
        print(self.vagrant.keyfile(vm_name=name), file=self.stdout)

    def version(self, **kwargs):
        print(self.vagrant.version(), file=self.stdout)

    def box_list(self, **kwargs):
        print(self.vagrant.box_list(), file=self.stdout)

    def snapshot_push(self, **kwargs):
        self.vagrant.snapshot_push()

    def snapshot_pop(self, **kwargs):
        self.vagrant.snapshot_pop()

    def snapshot_save(self, arguments, **kwargs):
        self.vagrant.snapshot_save(name=arguments[0])

    def snapshot_restore(self, arguments, **kwargs):
        self.vagrant.snapshot_restore(name=arguments[0])

    def snapshot_delete(self, arguments, **kwargs):
        self.vagrant.snapshot_delete(name=arguments[0])

    def snapshot_list(self, **kwargs):
        print(self.vagrant.snapshot_list(), file=self.stdout)

    def run_command(self, command, *args, **options):
        try:
            handler = self.commands[command]
        except KeyError:
            raise self.Error(
                'Unknown command: {0}.  Try --help?'.format(command))
        else:
            return handler(arguments=args, **options)

    def pretty_json(self, obj):
        json.dump(
            obj, self.stdout,
            sort_keys=True, indent=4, separators=(',', ': '),
        )
        print(file=self.stdout)

    def get_options(self):
        return (
            Option('--root', default=path(),
                   help='Directory holding Vagrantfile.'),
            Option('--name', default=None,
                   help='Optional VM name.'),
            Option('--provision-with', default=None,
                   type='string', action='callback',
                   callback=csv_list_option,
                   help='Optional comma-separated list of provisioners.'),
            Option('--quiet-stdout', action='store_true',
                   help='Disable output to stdout.'),
            Option('--quiet-stderr', action='store_true',
                   help='Disable output to stderr.'),
            Option('--force', action='store_true',
                   help='Force action (applies to e.g. halt).'),
        )


def main(argv=None):
    return vagrant(app='cyanide.app:app').execute_from_commandline(argv=argv)


if __name__ == '__main__':
    main()



from __future__ import absolute_import, print_function, unicode_literals

import celery

from celery import signals
from celery.bin.base import Option

from .templates import use_template, template_names

IS_CELERY_4 = celery.VERSION[0] >= 4


class App(celery.Celery):
    cyanide_suite = 'cyanide.suites.default:Default'
    template_selected = False

    def __init__(self, *args, **kwargs):
        self.template = kwargs.pop('template', None)
        super(App, self).__init__(*args, **kwargs)
        self.user_options['preload'].add(
            Option(
                '-Z', '--template', default='default', type=str,
                help='Configuration template to use: {0}'.format(
                    template_names(),
                ),
            )
        )
        signals.user_preload_options.connect(self.on_preload_parsed)
        if IS_CELERY_4:
            self.on_configure.connect(self._maybe_use_default_template)

    def on_preload_parsed(self, options=None, **kwargs):
        self.use_template(options['template'])

    def use_template(self, name='default'):
        if self.template_selected:
            raise RuntimeError('App already configured')
        self.template_selected = True
        use_template(self, name)

    def _maybe_use_default_template(self, **kwargs):
        if not self.template_selected:
            self.use_template('default')

    if not IS_CELERY_4:
        after_configure = None

        def _get_config(self):
            ret = super(App, self)._get_config()
            if self.after_configure:
                self.after_configure(ret)
            return ret

        def on_configure(self):
            self._maybe_use_default_template()

app = App('cyanide', set_as_current=False)



from __future__ import absolute_import, print_function, unicode_literals

from celery.bin.base import Command, Option
from celery.utils.imports import symbol_by_name

from cyanide.app import app as cyanide_app


class cyanide(Command):

    def __init__(self, app=None, *args, **kwargs):
        if app is None or app.main == 'default':
            app = cyanide_app
        app.set_current()
        app.set_default()
        super(cyanide, self).__init__(app, *args, **kwargs)

    def run(self, *names, **options):
        try:
            return self.run_suite(names, **options)
        except KeyboardInterrupt:
            print('###interrupted by user: exiting...', file=self.stdout)

    def run_suite(self, names, suite,
                  block_timeout=None, no_color=False, **options):
        return symbol_by_name(suite)(
            self.app,
            block_timeout=block_timeout,
            no_color=no_color,
        ).run(names, **options)

    def get_options(self):
        return (
            Option('-i', '--iterations', type='int', default=50,
                   help='Number of iterations for each test'),
            Option('-n', '--numtests', type='int', default=None,
                   help='Number of tests to execute'),
            Option('-o', '--offset', type='int', default=0,
                   help='Start at custom offset'),
            Option('--block-timeout', type='int', default=30 * 60),
            Option('-l', '--list', action='store_true', dest='list_all',
                   default=False, help='List all tests'),
            Option('-r', '--repeat', type='float', default=0,
                   help='Number of times to repeat the test suite'),
            Option('-g', '--group', default='all',
                   help='Specify test group (all|green|redis)'),
            Option('--diag', default=False, action='store_true',
                   help='Enable diagnostics (slow)'),
            Option('-J', '--no-join', default=False, action='store_true',
                   help='Do not wait for task results'),
            Option('-S', '--suite',
                   default=self.app.cyanide_suite,
                   help='Specify test suite to execute (path to class)'),
        )


def main(argv=None):
    return cyanide().execute_from_commandline(argv=argv)


if __name__ == '__main__':
    main()



from __future__ import absolute_import, unicode_literals

import celery
import os

from functools import partial

from celery.five import items
from kombu import Queue
from kombu.utils import symbol_by_name

CYANIDE_TRANS = os.environ.get('CYANIDE_TRANS', False)
default_queue = 'c.stress.trans' if CYANIDE_TRANS else 'c.stress'
CYANIDE_QUEUE = os.environ.get('CYANIDE_QUEUE', default_queue)

templates = {}

IS_CELERY_4 = celery.VERSION[0] >= 4


def template(name=None):

    def _register(cls):
        templates[name or cls.__name__] = '.'.join([__name__, cls.__name__])
        return cls
    return _register


if IS_CELERY_4:

    def use_template(app, template='default'):
        if isinstance(template, list) and len(template) == 1:
            # weird argparse thing
            template = template[0]
        template = template.split(',')

        # mixin the rest of the templates when the config is needed
        @app.on_after_configure.connect(weak=False)
        def load_template(sender, source, **kwargs):
            mixin_templates(template[1:], source)

        app.config_from_object(templates[template[0]])
else:

    def use_template(app, template='default'):  # noqa
        template = template.split(',')
        app.after_configure = partial(mixin_templates, template[1:])
        app.config_from_object(templates[template[0]])


def mixin_templates(templates, conf):
    return [mixin_template(template, conf) for template in templates]


def mixin_template(template, conf):
    cls = symbol_by_name(templates[template])
    conf.update(dict(
        (k, v) for k, v in items(vars(cls))
        if not k.startswith('_')
    ))


def template_names():
    return ', '.join(templates)


@template()
class default(object):
    CELERY_ACCEPT_CONTENT = ['json']
    BROKER_URL = os.environ.get('CYANIDE_BROKER', 'pyamqp://')
    BROKER_HEARTBEAT = 30
    CELERY_RESULT_BACKEND = os.environ.get('CYANIDE_BACKEND', 'rpc://')
    CELERY_RESULT_SERIALIZER = 'json'
    CELERY_RESULT_PERSISTENT = True
    CELERY_RESULT_EXPIRES = 300
    CELERY_MAX_CACHED_RESULTS = 100
    CELERY_DEFAULT_QUEUE = CYANIDE_QUEUE
    CELERY_IMPORTS = ['cyanide.tasks']
    CELERY_TRACK_STARTED = True
    CELERY_QUEUES = [
        Queue(CYANIDE_QUEUE,
              durable=not CYANIDE_TRANS,
              no_ack=CYANIDE_TRANS),
    ]
    CELERY_TASK_SERIALIZER = 'json'
    CELERY_TASK_PUBLISH_RETRY_POLICY = {
        'max_retries': 100,
        'interval_max': 2,
        'interval_step': 0.1,
    }
    CELERY_TASK_PROTOCOL = 2
    if CYANIDE_TRANS:
        CELERY_DEFAULT_DELIVERY_MODE = 1
    CELERYD_PREFETCH_MULTIPLIER = int(os.environ.get('CYANIDE_PREFETCH', 10))


@template()
class redis(default):
    BROKER_URL = os.environ.get('CYANIDE_BROKER', 'redis://')
    BROKER_TRANSPORT_OPTIONS = {
        'fanout_prefix': True,
        'fanout_patterns': True,
    }
    CELERY_RESULT_BACKEND = os.environ.get('CYANIDE_BACKEND', 'redis://')


@template()
class redistore(default):
    CELERY_RESULT_BACKEND = 'redis://'


@template()
class acks_late(default):
    CELERY_ACKS_LATE = True


@template()
class pickle(default):
    CELERY_ACCEPT_CONTENT = ['pickle', 'json']
    CELERY_TASK_SERIALIZER = 'pickle'
    CELERY_RESULT_SERIALIZER = 'pickle'


@template()
class confirms(default):
    BROKER_URL = 'pyamqp://'
    BROKER_TRANSPORT_OPTIONS = {'confirm_publish': True}


@template()
class events(default):
    CELERY_SEND_EVENTS = True
    CELERY_SEND_TASK_SENT_EVENT = True


@template()
class execv(default):
    CELERYD_FORCE_EXECV = True


@template()
class sqs(default):
    BROKER_URL = 'sqs://'
    BROKER_TRANSPORT_OPTIONS = {
        'region': os.environ.get('AWS_REGION', 'us-east-1'),
    }


@template()
class proto1(default):
    CELERY_TASK_PROTOCOL = 1


@template()
class vagrant1(default):
    BROKER_URL = 'pyamqp://testing:t3s71ng@192.168.33.123//testing'


@template()
class vagrant1_redis(redis):
    BROKER_URL = 'redis://192.168.33.123'
    CELERY_RESULT_BACKEND = 'redis://192.168.33.123'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from .compat import bytes_if_py2

try:
    import simplejson as json
except ImportError:
    import json  # noqa

type_registry = {}


class JSONEncoder(json.JSONEncoder):

    def default(self, obj):
        try:
            return super(JSONEncoder, self).default(obj)
        except TypeError:
            reducer = getattr(obj, '__to_json__', None)
            if reducer:
                return reducer()
            raise


def decode_hook(d):
    try:
        d = d['py/obj']
    except KeyError:
        return d
    type_registry[d['type']](**d['attrs'])


def install_json():
    json._default_encoder = JSONEncoder()
    json._default_decoder.object_hook = decode_hook
    try:
        from kombu.utils import json as kombujson
    except ImportError:
        pass
    else:
        kombujson._default_encoder = JSONEncoder
install_json()  # ugh, ugly but it's a test suite after all


# this imports kombu.utils.json, so can only import after install_json()
from celery.utils.debug import humanbytes  # noqa
from celery.utils.imports import qualname  # noqa


def json_reduce(obj, attrs):
    return {'py/obj': {'type': qualname(obj), 'attrs': attrs}}


def jsonable(cls):
    type_registry[qualname(cls)] = cls.__from_json__
    return cls


@jsonable
class Data(object):

    def __init__(self, label, data):
        self.label = label
        self.data = data

    def __str__(self):
        return bytes_if_py2('<Data: {0} ({1})>'.format(
            self.label, humanbytes(len(self.data)),
        ))

    def __repr__(self):
        return str(self)

    def __to_json__(self):
        return json_reduce(self, {'label': self.label, 'data': self.data})

    @classmethod
    def __from_json__(cls, label=None, data=None, **kwargs):
        return cls(label, data)

    def __reduce__(self):
        return Data, (self.label, self.data)

BIG = Data('BIG', 'x' * 2 ** 20 * 8)
SMALL = Data('SMALL', 'e' * 1024)



from __future__ import absolute_import, unicode_literals

import random

from time import sleep

from celery import group

from cyanide.tasks import (
    add, any_, exiting, kill, sleeping,
    sleeping_ignore_limits, any_returning,
)
from cyanide.data import BIG, SMALL
from cyanide.suite import Suite, testcase


class Default(Suite):

    @testcase('all', 'green')
    def manyshort(self):
        self.join(group(add.s(i, i) for i in range(1000))(),
                  timeout=10, propagate=True)

    @testcase('all')
    def always_timeout(self):
        self.join(
            group(sleeping.s(1).set(time_limit=0.1)
                  for _ in range(100))(),
            timeout=10, propagate=False,
        )

    @testcase('all')
    def termbysig(self):
        self._evil_groupmember(kill)

    @testcase('green')
    def group_with_exit(self):
        self._evil_groupmember(exiting)

    @testcase('all')
    def timelimits(self):
        self._evil_groupmember(sleeping, 2, time_limit=1)

    @testcase('all')
    def timelimits_soft(self):
        self._evil_groupmember(sleeping_ignore_limits, 2,
                               soft_time_limit=1, time_limit=1.1)

    @testcase('all')
    def alwayskilled(self):
        g = group(kill.s() for _ in range(10))
        self.join(g(), timeout=10)

    @testcase('all', 'green')
    def alwaysexits(self):
        g = group(exiting.s() for _ in range(10))
        self.join(g(), timeout=10)

    def _evil_groupmember(self, evil_t, *eargs, **opts):
        g1 = group(add.s(2, 2).set(**opts), evil_t.s(*eargs).set(**opts),
                   add.s(4, 4).set(**opts), add.s(8, 8).set(**opts))
        g2 = group(add.s(3, 3).set(**opts), add.s(5, 5).set(**opts),
                   evil_t.s(*eargs).set(**opts), add.s(7, 7).set(**opts))
        self.join(g1(), timeout=10)
        self.join(g2(), timeout=10)

    @testcase('all', 'green')
    def bigtasksbigvalue(self):
        g = group(any_returning.s(BIG, sleep=0.3) for i in range(8))
        r = g()
        try:
            self.join(r, timeout=10)
        finally:
            # very big values so remove results from backend
            try:
                r.forget()
            except NotImplementedError:
                pass

    @testcase('all', 'green')
    def bigtasks(self, wait=None):
        self._revoketerm(wait, False, False, BIG)

    @testcase('all', 'green')
    def smalltasks(self, wait=None):
        self._revoketerm(wait, False, False, SMALL)

    @testcase('all')
    def revoketermfast(self, wait=None):
        self._revoketerm(wait, True, False, SMALL)

    @testcase('all')
    def revoketermslow(self, wait=5):
        self._revoketerm(wait, True, True, BIG)

    def _revoketerm(self, wait=None, terminate=True,
                    joindelay=True, data=BIG):
        g = group(any_.s(data, sleep=wait) for i in range(8))
        r = g()
        if terminate:
            if joindelay:
                sleep(random.choice(range(4)))
            r.revoke(terminate=True)
        self.join(r, timeout=10)



from __future__ import absolute_import, print_function, unicode_literals

import socket
import sys

from contextlib import contextmanager

from celery import states


class FBI(object):

    def __init__(self, app):
        self.app = app
        self.receiver = None
        self.state = self.app.events.State()
        self.connection = None
        self.enabled = False

    def enable(self, enabled):
        self.enabled = enabled

    @contextmanager
    def investigation(self):
        if self.enabled:
            with self.app.connection() as conn:
                receiver = self.app.events.Receiver(
                    conn, handlers={'*': self.state.event},
                )
                with receiver.consumer_context() as (conn, _, _):
                    self.connection = conn
                    try:
                        yield self
                    finally:
                        self.ffwd()
        else:
            yield

    def ffwd(self):
        while 1:
            try:
                self.connection.drain_events(timeout=1)
            except socket.error:
                break

    def state_of(self, tid):
        try:
            task = self.state.tasks[tid]
        except KeyError:
            return 'No events for {0}'.format(tid)

        if task.state in states.READY_STATES:
            return 'Task {0.uuid} completed with {0.state}'.format(task)
        elif task.state in states.UNREADY_STATES:
            return 'Task {0.uuid} waiting in {0.state} state'.format(task)
        else:
            return 'Task {0.uuid} in other state {0.state}'.format(task)

    def query(self, ids):
        return self.app.control.inspect().query_task(id)

    def diag(self, ids, file=sys.stderr):
        if self.enabled:
            self.ffwd()
            for tid in ids:
                print(self.state_of(tid), file=file)



from __future__ import absolute_import, unicode_literals

import sys

__all__ = ['bytes_if_py2']

PY3 = sys.version_info[0] >= 3

if PY3:  # pragma: no cover
    def bytes_if_py2(s):
        return s
else:  # pragma: no cover
    def bytes_if_py2(s):  # noqa
        if isinstance(s, unicode):
            return s.encode()
        return s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import os
import re
import sys
import codecs

try:
    import platform
    _pyimp = platform.python_implementation
except (AttributeError, ImportError):
    def _pyimp():
        return 'Python'

NAME = 'cyanide'

E_UNSUPPORTED_PYTHON = '%s 1.0 requires %%s %%s or later!' % (NAME,)

PYIMP = _pyimp()
PY26_OR_LESS = sys.version_info < (2, 7)
PY3 = sys.version_info[0] == 3
PY33_OR_LESS = PY3 and sys.version_info < (3, 4)
PYPY_VERSION = getattr(sys, 'pypy_version_info', None)
PYPY = PYPY_VERSION is not None
PYPY24_ATLEAST = PYPY_VERSION and PYPY_VERSION >= (2, 4)

if PY26_OR_LESS:
    raise Exception(E_UNSUPPORTED_PYTHON % (PYIMP, '2.7'))
elif PY33_OR_LESS and not PYPY24_ATLEAST:
    raise Exception(E_UNSUPPORTED_PYTHON % (PYIMP, '3.4'))

# -*- Classifiers -*-

classes = """
    Development Status :: 2 - Pre-Alpha
    License :: OSI Approved :: BSD License
    Programming Language :: Python
    Programming Language :: Python :: 2
    Programming Language :: Python :: 2.7
    Programming Language :: Python :: 3
    Programming Language :: Python :: 3.4
    Programming Language :: Python :: 3.5
    Programming Language :: Python :: Implementation :: CPython
    Programming Language :: Python :: Implementation :: PyPy
    Programming Language :: Python :: Implementation :: Jython
    Operating System :: OS Independent
"""
classifiers = [s.strip() for s in classes.split('\n') if s]

# -*- Distribution Meta -*-

re_meta = re.compile(r'__(\w+?)__\s*=\s*(.*)')
re_doc = re.compile(r'^"""(.+?)"""')


def add_default(m):
    attr_name, attr_value = m.groups()
    return ((attr_name, attr_value.strip("\"'")),)


def add_doc(m):
    return (('doc', m.groups()[0]),)

pats = {re_meta: add_default, re_doc: add_doc}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, NAME, '__init__.py')) as fh:
    meta = {}
    for line in fh:
        if line.strip() == '# -eof meta-':
            break
        for pattern, handler in pats.items():
            m = pattern.match(line.strip())
            if m:
                meta.update(handler(m))

# -*- Installation Requires -*-


def strip_comments(l):
    return l.split('#', 1)[0].strip()


def _pip_requirement(req):
    if req.startswith('-r '):
        _, path = req.split()
        return reqs(*path.split('/'))
    return [req]


def _reqs(*f):
    return [
        _pip_requirement(r) for r in (
            strip_comments(l) for l in open(
                os.path.join(os.getcwd(), 'requirements', *f)).readlines()
        ) if r]


def reqs(*f):
    return [req for subreq in _reqs(*f) for req in subreq]

# -*- Long Description -*-

if os.path.exists('README.rst'):
    long_description = codecs.open('README.rst', 'r', 'utf-8').read()
else:
    long_description = 'See http://pypi.python.org/pypi/%s' % (NAME,)

# -*- %%% -*-

setup(
    name=NAME,
    version=meta['version'],
    description=meta['doc'],
    keywords='celery integration test suite',
    author=meta['author'],
    author_email=meta['contact'],
    url=meta['homepage'],
    platforms=['any'],
    license='BSD',
    packages=find_packages(exclude=['ez_setup', 'tests', 'tests.*']),
    include_package_data=True,
    zip_safe=False,
    install_requires=reqs('default.txt'),
    tests_require=reqs('test.txt'),
    test_suite='nose.collector',
    classifiers=classifiers,
    long_description=long_description,
    entry_points={
        'celery.commands': [
            'cyanide = cyanide.bin.cyanide:cyanide',
            'vagrant = cyanide.bin.vagrant:vagrant',
        ],
    },
)



from __future__ import absolute_import, unicode_literals

import os
import vagrant


def path():
    return os.path.abspath(os.path.dirname(__file__))


class Vagrant(vagrant.Vagrant):

    def __init__(self, root=None, quiet_stdout=False, quiet_stderr=False,
                 *args, **kwargs):
        super(Vagrant, self).__init__(
            root or path(),
            quiet_stdout=quiet_stdout,
            quiet_stderr=quiet_stderr,
            *args, **kwargs)
